@@ -1,5 +1,5 @@
 /* C language support definitions for GDB, the GNU debugger.
-   Copyright 1992, 1996 Free Software Foundation, Inc.
+   Copyright 1992, 1996, 2000 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -31,23 +31,24 @@ c_parse PARAMS ((void));	/* Defined in c-exp.y */
 extern void
 c_error PARAMS ((char *));	/* Defined in c-exp.y */
 
-extern void			/* Defined in c-typeprint.c */
-c_print_type PARAMS ((struct type *, char *, GDB_FILE *, int, int));
+/* Defined in c-typeprint.c */
+extern void c_print_type (struct type *, char *, struct ui_file *, int,
+			  int);
 
-extern int
-c_val_print PARAMS ((struct type *, char *, int, CORE_ADDR, GDB_FILE *, int, int,
-		     int, enum val_prettyprint));
+extern int c_val_print (struct type *, char *, int, CORE_ADDR,
+			struct ui_file *, int, int, int,
+			enum val_prettyprint);
 
-extern int
-c_value_print PARAMS ((struct value *, GDB_FILE *, int, enum val_prettyprint));
+extern int c_value_print (struct value *, struct ui_file *, int,
+			  enum val_prettyprint);
 
 /* These are in c-lang.c: */
 
-extern void c_printchar PARAMS ((int, GDB_FILE *));
+extern void c_printchar (int, struct ui_file *);
 
-extern void c_printstr PARAMS ((GDB_FILE * stream, char *string,
-				unsigned int length, int width,
-				int force_ellipses));
+extern void c_printstr (struct ui_file * stream, char *string,
+			unsigned int length, int width,
+			int force_ellipses);
 
 extern struct type *c_create_fundamental_type PARAMS ((struct objfile *, int));
 
@@ -55,11 +56,10 @@ extern struct type **CONST_PTR (c_builtin_types[]);
 
 /* These are in c-typeprint.c: */
 
-extern void
-c_type_print_base PARAMS ((struct type *, GDB_FILE *, int, int));
+extern void c_type_print_base (struct type *, struct ui_file *, int, int);
 
-extern void
-c_type_print_varspec_prefix PARAMS ((struct type *, GDB_FILE *, int, int));
+extern void c_type_print_varspec_prefix (struct type *, struct ui_file *,
+					 int, int);
 
 /* These are in cp-valprint.c */
 
@@ -67,16 +67,15 @@ extern int vtblprint;		/* Controls printing of vtbl's */
 
 extern int static_field_print;
 
-extern void
-cp_print_class_member PARAMS ((char *, struct type *, GDB_FILE *, char *));
+extern void cp_print_class_member (char *, struct type *, struct ui_file *,
+				   char *);
 
-extern void
-cp_print_class_method PARAMS ((char *, struct type *, GDB_FILE *));
+extern void cp_print_class_method (char *, struct type *, struct ui_file *);
 
-extern void
-cp_print_value_fields PARAMS ((struct type *, struct type *, char *, int, CORE_ADDR,
-			       GDB_FILE *, int, int, enum val_prettyprint,
-			       struct type **, int));
+extern void cp_print_value_fields (struct type *, struct type *, char *,
+				   int, CORE_ADDR, struct ui_file *, int,
+				   int, enum val_prettyprint,
+				   struct type **, int);
 
 extern int
 cp_is_vtbl_ptr_type PARAMS ((struct type *));
