@@ -1,5 +1,5 @@
 /* Native support for GNU/Linux, for GDB, the GNU debugger.
-   Copyright (C) 1986, 1987, 1989, 1992, 1996, 1998
+   Copyright (C) 1986, 1987, 1989, 1992, 1996, 1998, 2000
    Free Software Foundation, Inc.
 
    This file is part of GDB.
@@ -23,15 +23,13 @@
 #define NM_LINUX_H
 
 #include "i386/nm-i386v.h"
+#include "nm-linux.h"
 
 /* Return sizeof user struct to callers in less machine dependent routines */
 
 #define KERNEL_U_SIZE kernel_u_size()
 extern int kernel_u_size PARAMS ((void));
 
-/* Tell gdb that we can attach and detach other processes */
-#define ATTACH_DETACH
-
 #define U_REGS_OFFSET 0
 
 /* GNU/Linux supports the 386 hardware debugging registers.  */
@@ -64,9 +62,16 @@ extern int kernel_u_size PARAMS ((void));
 #include "solib.h"		/* Support for shared libraries. */
 #endif
 
-/* Override copies of {fetch,store}_inferior_registers in infptrace.c.  */
+/* Override copies of {fetch,store}_inferior_registers in `infptrace.c'.  */
 #define FETCH_INFERIOR_REGISTERS
 
+/* Nevertheless, define CANNOT_{FETCH,STORE}_REGISTER, because we fall
+   back on the code `infptrace.c' (well a copy of that code in
+   `i386-linux-nat.c' for now) and we can access only the
+   general-purpose registers in that way.  */
+#define CANNOT_FETCH_REGISTER(regno) ((regno) >= NUM_GREGS)
+#define CANNOT_STORE_REGISTER(regno) CANNOT_FETCH_REGISTER (regno)
+
 extern CORE_ADDR
   i386_stopped_by_watchpoint PARAMS ((int));
 extern int
@@ -74,22 +79,4 @@ i386_insert_watchpoint PARAMS ((int pid, CORE_ADDR addr, int len, int rw));
 extern int
 i386_remove_watchpoint PARAMS ((int pid, CORE_ADDR addr, int len));
 
-/* Support for the glibc linuxthreads package. */
-
-#ifdef __STDC__
-struct objfile;
-#endif
-
-extern void
-linuxthreads_new_objfile PARAMS ((struct objfile *objfile));
-#define target_new_objfile(OBJFILE) linuxthreads_new_objfile (OBJFILE)
-
-extern char *
-linuxthreads_pid_to_str PARAMS ((int pid));
-#define target_pid_to_str(PID) linuxthreads_pid_to_str (PID)
-
-extern int
-linuxthreads_prepare_to_proceed PARAMS ((int step));
-#define PREPARE_TO_PROCEED(select_it) linuxthreads_prepare_to_proceed (1)
-
 #endif /* #ifndef NM_LINUX_H */
