@@ -57,10 +57,8 @@ static int read_relative_register_raw_bytes_for_frame PARAMS ((int regnum, char
 you lose
 #endif
 
-  LONGEST
-extract_signed_integer (addr, len)
-     PTR addr;
-     int len;
+LONGEST
+extract_signed_integer (void *addr, int len)
 {
   LONGEST retval;
   unsigned char *p;
@@ -94,9 +92,7 @@ That operation is not available on integers of more than %d bytes.",
 }
 
 ULONGEST
-extract_unsigned_integer (addr, len)
-     PTR addr;
-     int len;
+extract_unsigned_integer (void *addr, int len)
 {
   ULONGEST retval;
   unsigned char *p;
@@ -130,10 +126,7 @@ That operation is not available on integers of more than %d bytes.",
    function returns 1 and sets *PVAL.  Otherwise it returns 0.  */
 
 int
-extract_long_unsigned_integer (addr, orig_len, pval)
-     PTR addr;
-     int orig_len;
-     LONGEST *pval;
+extract_long_unsigned_integer (void *addr, int orig_len, LONGEST *pval)
 {
   char *p, *first_addr;
   int len;
@@ -177,9 +170,7 @@ extract_long_unsigned_integer (addr, orig_len, pval)
 }
 
 CORE_ADDR
-extract_address (addr, len)
-     PTR addr;
-     int len;
+extract_address (void *addr, int len)
 {
   /* Assume a CORE_ADDR can fit in a LONGEST (for now).  Not sure
      whether we want this to be true eventually.  */
@@ -187,10 +178,7 @@ extract_address (addr, len)
 }
 
 void
-store_signed_integer (addr, len, val)
-     PTR addr;
-     int len;
-     LONGEST val;
+store_signed_integer (void *addr, int len, LONGEST val)
 {
   unsigned char *p;
   unsigned char *startaddr = (unsigned char *) addr;
@@ -217,10 +205,7 @@ store_signed_integer (addr, len, val)
 }
 
 void
-store_unsigned_integer (addr, len, val)
-     PTR addr;
-     int len;
-     ULONGEST val;
+store_unsigned_integer (void *addr, int len, ULONGEST val)
 {
   unsigned char *p;
   unsigned char *startaddr = (unsigned char *) addr;
@@ -250,33 +235,11 @@ store_unsigned_integer (addr, len, val)
    gdb-local memory pointed to by "addr"
    for "len" bytes. */
 void
-store_address (addr, len, val)
-     PTR addr;
-     int len;
-     LONGEST val;
+store_address (void *addr, int len, LONGEST val)
 {
   store_unsigned_integer (addr, len, val);
 }
 
-/* Swap LEN bytes at BUFFER between target and host byte-order.  */
-#define SWAP_FLOATING(buffer,len) \
-  do                                                                    \
-    {                                                                   \
-      if (TARGET_BYTE_ORDER != HOST_BYTE_ORDER)                         \
-        {                                                               \
-          char tmp;                                                     \
-          char *p = (char *)(buffer);                                   \
-          char *q = ((char *)(buffer)) + len - 1;                       \
-          for (; p < q; p++, q--)                                       \
-            {                                                           \
-              tmp = *q;                                                 \
-              *q = *p;                                                  \
-              *p = tmp;                                                 \
-            }                                                           \
-        }                                                               \
-    }                                                                   \
-  while (0)
-
 /* Extract a floating-point number from a target-order byte-stream at ADDR.
    Returns the value as type DOUBLEST.
 
@@ -286,13 +249,11 @@ store_address (addr, len, val)
    dirty work.  */
 
 DOUBLEST
-extract_floating (addr, len)
-     PTR addr;
-     int len;
+extract_floating (void *addr, int len)
 {
   DOUBLEST dretval;
 
-  if (len == sizeof (float))
+  if (len * TARGET_CHAR_BIT == TARGET_FLOAT_BIT)
     {
       if (HOST_FLOAT_FORMAT == TARGET_FLOAT_FORMAT)
 	{
@@ -304,7 +265,7 @@ extract_floating (addr, len)
       else
 	floatformat_to_doublest (TARGET_FLOAT_FORMAT, addr, &dretval);
     }
-  else if (len == sizeof (double))
+  else if (len * TARGET_CHAR_BIT == TARGET_DOUBLE_BIT)
     {
       if (HOST_DOUBLE_FORMAT == TARGET_DOUBLE_FORMAT)
 	{
@@ -316,7 +277,7 @@ extract_floating (addr, len)
       else
 	floatformat_to_doublest (TARGET_DOUBLE_FORMAT, addr, &dretval);
     }
-  else if (len == sizeof (DOUBLEST))
+  else if (len * TARGET_CHAR_BIT == TARGET_LONG_DOUBLE_BIT)
     {
       if (HOST_LONG_DOUBLE_FORMAT == TARGET_LONG_DOUBLE_FORMAT)
 	{
@@ -328,10 +289,6 @@ extract_floating (addr, len)
       else
 	floatformat_to_doublest (TARGET_LONG_DOUBLE_FORMAT, addr, &dretval);
     }
-#ifdef TARGET_EXTRACT_FLOATING
-  else if (TARGET_EXTRACT_FLOATING (addr, len, &dretval))
-    return dretval;
-#endif
   else
     {
       error ("Can't deal with a floating point number of %d bytes.", len);
@@ -341,12 +298,9 @@ extract_floating (addr, len)
 }
 
 void
-store_floating (addr, len, val)
-     PTR addr;
-     int len;
-     DOUBLEST val;
+store_floating (void *addr, int len, DOUBLEST val)
 {
-  if (len == sizeof (float))
+  if (len * TARGET_CHAR_BIT == TARGET_FLOAT_BIT)
     {
       if (HOST_FLOAT_FORMAT == TARGET_FLOAT_FORMAT)
 	{
@@ -357,7 +311,7 @@ store_floating (addr, len, val)
       else
 	floatformat_from_doublest (TARGET_FLOAT_FORMAT, &val, addr);
     }
-  else if (len == sizeof (double))
+  else if (len * TARGET_CHAR_BIT == TARGET_DOUBLE_BIT)
     {
       if (HOST_DOUBLE_FORMAT == TARGET_DOUBLE_FORMAT)
 	{
@@ -368,17 +322,13 @@ store_floating (addr, len, val)
       else
 	floatformat_from_doublest (TARGET_DOUBLE_FORMAT, &val, addr);
     }
-  else if (len == sizeof (DOUBLEST))
+  else if (len * TARGET_CHAR_BIT == TARGET_LONG_DOUBLE_BIT)
     {
       if (HOST_LONG_DOUBLE_FORMAT == TARGET_LONG_DOUBLE_FORMAT)
 	memcpy (addr, &val, sizeof (val));
       else
 	floatformat_from_doublest (TARGET_LONG_DOUBLE_FORMAT, &val, addr);
     }
-#ifdef TARGET_STORE_FLOATING
-  else if (TARGET_STORE_FLOATING (addr, len, val))
-    return;
-#endif 
   else
     {
       error ("Can't deal with a floating point number of %d bytes.", len);
@@ -680,20 +630,22 @@ registers_fetched ()
     register_valid[i] = 1;
 }
 
-/* read_register_bytes and write_register_bytes are generally a *BAD* idea.
-   They are inefficient because they need to check for partial updates, which
-   can only be done by scanning through all of the registers and seeing if the
-   bytes that are being read/written fall inside of an invalid register.  [The
-   main reason this is necessary is that register sizes can vary, so a simple
-   index won't suffice.]  It is far better to call read_register_gen if you
-   want to get at the raw register contents, as it only takes a regno as an
-   argument, and therefore can't do a partial register update.  It would also
-   be good to have a write_register_gen for similar reasons.
+/* read_register_bytes and write_register_bytes are generally a *BAD*
+   idea.  They are inefficient because they need to check for partial
+   updates, which can only be done by scanning through all of the
+   registers and seeing if the bytes that are being read/written fall
+   inside of an invalid register.  [The main reason this is necessary
+   is that register sizes can vary, so a simple index won't suffice.]
+   It is far better to call read_register_gen and write_register_gen
+   if you want to get at the raw register contents, as it only takes a
+   regno as an argument, and therefore can't do a partial register
+   update.
 
-   Prior to the recent fixes to check for partial updates, both read and
-   write_register_bytes always checked to see if any registers were stale, and
-   then called target_fetch_registers (-1) to update the whole set.  This
-   caused really slowed things down for remote targets.  */
+   Prior to the recent fixes to check for partial updates, both read
+   and write_register_bytes always checked to see if any registers
+   were stale, and then called target_fetch_registers (-1) to update
+   the whole set.  This caused really slowed things down for remote
+   targets.  */
 
 /* Copy INLEN bytes of consecutive data from registers
    starting with the INREGBYTE'th byte of register data
@@ -720,7 +672,6 @@ read_register_bytes (inregbyte, myaddr, inlen)
   for (regno = 0; regno < NUM_REGS; regno++)
     {
       int regstart, regend;
-      int startin, endin;
 
       if (register_valid[regno])
 	continue;
@@ -731,15 +682,12 @@ read_register_bytes (inregbyte, myaddr, inlen)
       regstart = REGISTER_BYTE (regno);
       regend = regstart + REGISTER_RAW_SIZE (regno);
 
-      startin = regstart >= inregbyte && regstart < inregend;
-      endin = regend > inregbyte && regend <= inregend;
-
-      if (!startin && !endin)
+      if (regend <= inregbyte || inregend <= regstart)
+	/* The range the user wants to read doesn't overlap with regno.  */
 	continue;
 
       /* We've found an invalid register where at least one byte will be read.
          Update it from the target.  */
-
       target_fetch_registers (regno);
 
       if (!register_valid[regno])
@@ -832,40 +780,41 @@ write_register_bytes (myregstart, myaddr, inlen)
   for (regno = 0; regno < NUM_REGS; regno++)
     {
       int regstart, regend;
-      int startin, endin;
-      char regbuf[MAX_REGISTER_RAW_SIZE];
 
       regstart = REGISTER_BYTE (regno);
       regend = regstart + REGISTER_RAW_SIZE (regno);
 
-      startin = regstart >= myregstart && regstart < myregend;
-      endin = regend > myregstart && regend <= myregend;
+      /* Is this register completely outside the range the user is writing?  */
+      if (myregend <= regstart || regend <= myregstart)
+	/* do nothing */ ;		
 
-      if (!startin && !endin)
-	continue;		/* Register is completely out of range */
+      /* Is this register completely within the range the user is writing?  */
+      else if (myregstart <= regstart && regend <= myregend)
+	write_register_gen (regno, myaddr + (regstart - myregstart));
 
-      if (startin && endin)	/* register is completely in range */
+      /* The register partially overlaps the range being written.  */
+      else
 	{
-	  write_register_gen (regno, myaddr + (regstart - myregstart));
-	  continue;
+	  char regbuf[MAX_REGISTER_RAW_SIZE];
+	  /* What's the overlap between this register's bytes and
+             those the caller wants to write?  */
+	  int overlapstart = max (regstart, myregstart);
+	  int overlapend   = min (regend,   myregend);
+
+	  /* We may be doing a partial update of an invalid register.
+	     Update it from the target before scribbling on it.  */
+	  read_register_gen (regno, regbuf);
+
+	  memcpy (registers + overlapstart,
+		  myaddr + (overlapstart - myregstart),
+		  overlapend - overlapstart);
+
+	  target_store_registers (regno);
 	}
-
-      /* We may be doing a partial update of an invalid register.  Update it
-         from the target before scribbling on it.  */
-      read_register_gen (regno, regbuf);
-
-      if (startin)
-	memcpy (registers + regstart,
-		myaddr + regstart - myregstart,
-		myregend - regstart);
-      else			/* endin */
-	memcpy (registers + myregstart,
-		myaddr,
-		regend - myregstart);
-      target_store_registers (regno);
     }
 }
 
+
 /* Return the raw contents of register REGNO, regarding it as an integer.  */
 /* This probably should be returning LONGEST rather than CORE_ADDR.  */
 
@@ -1026,7 +975,7 @@ supply_register (regno, val)
 #endif
 
 CORE_ADDR
-generic_target_read_pc (pid)
+generic_target_read_pc (int pid)
 {
 #ifdef PC_REGNUM
   if (PC_REGNUM >= 0)
@@ -1474,6 +1423,10 @@ value_from_register (type, regnum, frame)
   CHECK_TYPEDEF (type);
   len = TYPE_LENGTH (type);
 
+  /* Pointers on D10V are really only 16 bits, but we lie to gdb elsewhere... */
+  if (GDB_TARGET_IS_D10V && TYPE_CODE (type) == TYPE_CODE_PTR)
+    len = 2;
+
   VALUE_REGNO (v) = regnum;
 
   num_storage_locs = (len > REGISTER_VIRTUAL_SIZE (regnum) ?
@@ -1672,6 +1625,29 @@ value_from_register (type, regnum, frame)
       memcpy (VALUE_CONTENTS_RAW (v), raw_buffer + VALUE_OFFSET (v), len);
     }
 
+  if (GDB_TARGET_IS_D10V
+      && TYPE_CODE (type) == TYPE_CODE_PTR
+      && TYPE_TARGET_TYPE (type)
+      && (TYPE_CODE (TYPE_TARGET_TYPE (type)) == TYPE_CODE_FUNC))
+    {
+      /* pointer to function */
+      unsigned long num;
+      unsigned short snum;
+      snum = (unsigned short) extract_unsigned_integer (VALUE_CONTENTS_RAW (v), 2);
+      num = D10V_MAKE_IADDR (snum);
+      store_address (VALUE_CONTENTS_RAW (v), 4, num);
+    }
+  else if (GDB_TARGET_IS_D10V
+	   && TYPE_CODE (type) == TYPE_CODE_PTR)
+    {
+      /* pointer to data */
+      unsigned long num;
+      unsigned short snum;
+      snum = (unsigned short) extract_unsigned_integer (VALUE_CONTENTS_RAW (v), 2);
+      num = D10V_MAKE_DADDR (snum);
+      store_address (VALUE_CONTENTS_RAW (v), 4, num);
+    }
+
   return v;
 }
 
