@@ -1,31 +1,32 @@
 /* sparc-dependent portions of the RPC protocol
    used with a VxWorks target 
 
-Contributed by Wind River Systems.
+   Contributed by Wind River Systems.
 
-This file is part of GDB.
+   This file is part of GDB.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.  */
 
 #include <stdio.h>
 #include "defs.h"
 
-#include "vx-share/regPacket.h"  
+#include "vx-share/regPacket.h"
 #include "frame.h"
 #include "inferior.h"
-#include "wait.h"
+#include "gdb_wait.h"
 #include "target.h"
 #include "gdbcore.h"
 #include "command.h"
@@ -40,7 +41,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 #include <sys/time.h>
 #include <sys/socket.h>
 
-#ifdef _AIX                     /* IBM claims "void *malloc()" not char * */
+#ifdef _AIX			/* IBM claims "void *malloc()" not char * */
 #define malloc bogon_malloc
 #endif
 
@@ -102,8 +103,8 @@ vx_read_register (regno)
      active frame automatically; it greatly simplifies debugging
      (FRAME_FIND_SAVED_REGS, in particular, depends on this).  */
 
-  sp = extract_address (&registers[REGISTER_BYTE (SP_REGNUM)], 
-	REGISTER_RAW_SIZE (CORE_ADDR));
+  sp = extract_address (&registers[REGISTER_BYTE (SP_REGNUM)],
+			REGISTER_RAW_SIZE (CORE_ADDR));
   write_memory (sp, &registers[REGISTER_BYTE (L0_REGNUM)],
 		16 * REGISTER_RAW_SIZE (L0_REGNUM));
 
@@ -115,14 +116,14 @@ vx_read_register (regno)
   if (target_has_fp)
     {
       net_read_registers (sparc_fpreg_packet, SPARC_FPREG_PLEN,
-                          PTRACE_GETFPREGS);
-      bcopy (&sparc_fpreg_packet[SPARC_R_FP0], 
-             &registers[REGISTER_BYTE (FP0_REGNUM)], 32 * SPARC_FPREG_SIZE);
+			  PTRACE_GETFPREGS);
+      bcopy (&sparc_fpreg_packet[SPARC_R_FP0],
+	     &registers[REGISTER_BYTE (FP0_REGNUM)], 32 * SPARC_FPREG_SIZE);
       bcopy (&sparc_fpreg_packet[SPARC_R_FSR],
 	     &registers[REGISTER_BYTE (FPS_REGNUM)], 1 * SPARC_FPREG_SIZE);
     }
   else
-    { 
+    {
       bzero (&registers[REGISTER_BYTE (FP0_REGNUM)], 32 * SPARC_FPREG_SIZE);
       bzero (&registers[REGISTER_BYTE (FPS_REGNUM)], 1 * SPARC_FPREG_SIZE);
     }
@@ -155,7 +156,7 @@ vx_write_register (regno)
   if (regno >= 0)
     {
       if ((G0_REGNUM <= regno && regno <= I7_REGNUM)
-          || (Y_REGNUM <= regno && regno <= NPC_REGNUM))
+	  || (Y_REGNUM <= regno && regno <= NPC_REGNUM))
 	in_fp_regs = 0;
       else
 	in_gp_regs = 0;
@@ -173,9 +174,9 @@ vx_write_register (regno)
          registers, update the register window spill area.  */
 
       if (regno < 0 || (L0_REGNUM <= regno && regno <= I7_REGNUM))
-        {
-  	  sp = extract_address (&registers[REGISTER_BYTE (SP_REGNUM)], 
-		REGISTER_RAW_SIZE (CORE_ADDR));
+	{
+	  sp = extract_address (&registers[REGISTER_BYTE (SP_REGNUM)],
+				REGISTER_RAW_SIZE (CORE_ADDR));
 	  write_memory (sp, &registers[REGISTER_BYTE (L0_REGNUM)],
 			16 * REGISTER_RAW_SIZE (L0_REGNUM));
 	}
@@ -185,12 +186,12 @@ vx_write_register (regno)
 
   if (in_fp_regs && target_has_fp)
     {
-      bcopy (&registers[REGISTER_BYTE (FP0_REGNUM)], 
+      bcopy (&registers[REGISTER_BYTE (FP0_REGNUM)],
 	     &sparc_fpreg_packet[SPARC_R_FP0], 32 * SPARC_FPREG_SIZE);
-      bcopy (&registers[REGISTER_BYTE (FPS_REGNUM)], 
+      bcopy (&registers[REGISTER_BYTE (FPS_REGNUM)],
 	     &sparc_fpreg_packet[SPARC_R_FSR], 1 * SPARC_FPREG_SIZE);
 
       net_write_registers (sparc_fpreg_packet, SPARC_FPREG_PLEN,
-                           PTRACE_SETFPREGS);
+			   PTRACE_SETFPREGS);
     }
 }
