@@ -88,9 +88,11 @@ static char *sh3e_reg_names[] = {
 };
 /* *INDENT-ON* */
 
-
-
+#ifdef _WIN32_WCE
+char **sh_register_names = sh3_reg_names;
+#else
 char **sh_register_names = sh_generic_reg_names;
+#endif
 
 struct
   {
@@ -103,6 +105,10 @@ sh_processor_type_table[] =
     sh_reg_names, bfd_mach_sh
   }
   ,
+  {
+    sh_reg_names, bfd_mach_sh2
+  }
+  ,
   {
     sh3_reg_names, bfd_mach_sh3
   }
@@ -111,10 +117,6 @@ sh_processor_type_table[] =
     sh3e_reg_names, bfd_mach_sh3e
   }
   ,
-  {
-    sh3e_reg_names, bfd_mach_sh4
-  }
-  ,
   {
     NULL, 0
   }
@@ -636,10 +638,6 @@ sh_show_regs (args, from_tty)
   else
     cpu = 0;
 
-  /* FIXME: sh4 has more registers */
-  if (cpu == bfd_mach_sh4)
-    cpu = bfd_mach_sh3;
-
   printf_filtered ("PC=%s SR=%08lx PR=%08lx MACH=%08lx MACHL=%08lx\n",
 		   paddr (read_register (PC_REGNUM)),
 		   (long) read_register (SR_REGNUM),
