@@ -65,6 +65,10 @@ enum {
  | (1 << PROFILE_MODEL_IDX) \
  | (1 << PROFILE_CORE_IDX))
 
+/* Utility to set profile options.  */
+SIM_RC set_profile_option_mask (SIM_DESC sd_, const char *name_, int mask_,
+				const char *arg_);
+
 /* Utility to parse a --profile-<foo> option.  */
 /* ??? On the one hand all calls could be confined to sim-profile.c, but
    on the other hand keeping a module's profiling option with the module's
